#!/bin/bash

CheckAvailability() {
local Address="$(sed -r -e 's/[^/]*\/\/([^@]*@)?([^:/]*).*/\2/' <<< $1)"
if ! ping -q -c 3 -W 3 "$Address" >/dev/null
then
	echo -e "\x1b[1;31mНет соединения с \"$Address\"\x1b[0m"
	exit 4
fi
}

PASSWORD="111"
while [[ $# > 0 ]]; do
    TEMP="$1"

    case $TEMP in
        -h|--help)
            echo "Параметры:"
            echo "	-h|-help        Помощь"
            echo "	-ip             Выполнять действия удалённо, по указанному адресу. Без этого параметра, скрипт работает на локальной машине"
			echo "	-p|-password    Пароль для SSH. По-умолчанию \"$PASSWORD\""
            echo "	-packages    	Список пакетов, с которыми нужно работать. По умолчанию тип кассы. Если тип кассы не удалось определить, artix-pos"
			echo -e "\nПример: ${0##*/} -ip 192.168.0.15 -packages \"artix-pos artix-servicestub\""
            exit 0
            ;;
        -ip)
			if [[ ! "$2" =~ ^(([1-9]?[0-9]|1[0-9][0-9]|2([0-4][0-9]|5[0-5]))\.){3}([1-9]?[0-9]|1[0-9][0-9]|2([0-4][0-9]|5[0-5]))$ ]]
			then
				echo -e "\x1b[1;31mНекорректное значение параметра \"$TEMP\" \"$2\"\x1b[0m"
				exit 1
			fi
			CheckAvailability "$2"
			IP="$2"
            shift
            ;;
		-p|-password)
			PASSWORD="$2"
            shift
            ;;
		-packages)
			PACKAGES=($2)
            shift
            ;;
        *)
			echo -e "\x1b[1;31mНеизвестный параметр \"$TEMP\"\x1b[0m"
			exit 1
        ;;
    esac
    shift
done

if [ -n "$IP" ] && ! sshpass -p "$PASSWORD" ssh -q root@$IP "exit"
then
	echo -e "\x1b[1;31mНет доступа\x1b[0m"
	exit 5
fi

ExecuteAction() {
if [ -z "$IP" ]
then
	eval "$1"
else
	eval "sshpass -p "$PASSWORD" ssh -q root@$IP "\"$1\"""
fi
return $?
}

#Получаем текущую версию, проверяем, что попали на кассу
#artix-syncagent используется потому, что имеет нормальное версионирование и есть в зависимостях других метапакетов
TEMP="$(ExecuteAction "dpkg -s artix-syncagent 2> /dev/null")"
TEMP="${TEMP#*Version: }"
CURRENT_VERSION="${TEMP%%$'\n'*}"
if [ -z "$CURRENT_VERSION" ]
then
	echo -e "\x1b[1;31mНе касса!\x1b[0m"
	exit 2
fi

VERSION="$(ExecuteAction "grep -h -R '^[^#].*artix' --include='*.list' '/etc/apt'")"
if [ $? -ne 0 ]
then
	echo -e "\x1b[1;31mНевозможно определить версию\x1b[0m"
	exit 1
fi
VERSION="${VERSION#* }"
VERSION="${VERSION#*] }"
REPOSITORY="${VERSION%% *}"
VERSION="${VERSION#* }"
VERSION="${VERSION%% *}"
VERSION="${VERSION#*-}"
if [[ ! "$VERSION" =~ ^(4\.6\.[1-9][0-9]*-[1-9][0-9]*)|devel$ ]]
then
	echo -e "\x1b[1;31mНекорректная версия \"$VERSION\"\x1b[0m"
	exit 1
fi

CheckAvailability "$REPOSITORY"

GetCurrentCashType() {
    local variants=("artix-pos" "artix-pos-sco" "artix-sst-sco")
    for variant in "${variants[@]}"; do
        if $(ExecuteAction "dpkg-query -s \"$variant\" >/dev/null 2>&1"); then
            echo $variant
            exit
        fi
    done
    echo 'artix-pos'
}

if [ ! -v PACKAGES ]; then
	PACKAGES=$(GetCurrentCashType )
fi

#Проверяем, есть ли указанная версия в репозитории
TEMP="$(curl -s $REPOSITORY/dists/)"
if [[ ! "$TEMP" =~ .*"$VERSION"[-/].* ]]
then
	echo -e "\x1b[1;31mВерсии \"$VERSION\" нет в репозитории \"$REPOSITORY\"\x1b[0m"
	exit 3
fi

echo -e "Текущая версия \x1b[1;32m\"$CURRENT_VERSION\"\x1b[0m, запрошенная версия \x1b[1;31m\"$VERSION\"\x1b[0m"
read -p "Продолжить (Enter)? " -n 1 TEMP
echo ""
if [ -z "$TEMP" ]
then
	ExecuteAction "apt-get update"
	if [ "$VERSION" == "devel" ]
	then
		VERSION="$(ExecuteAction "apt-cache madison artix-upgrade-system2")"
		VERSION="${VERSION#*|}"
		VERSION="${VERSION%%|*}"
		VERSION="${VERSION//[[:space:]]/}"
	fi
	for PACKAGE in "${PACKAGES[@]}"
	do
		TEMP="$TEMP\n$PACKAGE"
	done
	TEMP="${TEMP:2}"
	ExecuteAction "apt-get install artix-upgrade-system2=$VERSION -y --allow-unauthenticated --allow-downgrades && \
		touch /linuxcash/cash/data/upgrade-system/new-install && \
		echo -e '$TEMP' > /linuxcash/cash/conf/artix-packages.list && \
		cd /linuxcash/cash/bin && /linuxcash/cash/bin/artix-upgrade.sh -i -l '' && pkill artix; pkill sst-sco"
fi
