#!/usr/bin/env python3

import dialog
import subprocess
import hostname_util
import os
import sys
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.wizard.wizard import Wizard
from py_tools.logger import logger

log = logger.Logger()

class HostnameWizard(Wizard):
    """Класс визарда настройки hostname"""

    def __init__(self):
        """Конструктор визарда настройки hostname"""
        Wizard.__init__(self)
        self.hostname = ""
        self.hostname_filename = "/etc/hostname"
        self.hosts_filename = "/etc/hosts"
        self.d = dialog.Dialog()
        self.d.add_persistent_args(("--backtitle", "Настройка имени хоста"))

        #настройки wizard'а
        self.dialogs = [
            {
                'name': 'setup_hostname',
                'function': lambda: self.setup_hostname(),
                'back': None,
                'next': 'finish'
            },
            {
                'name': 'finish',
                'function': lambda: self.finish(),
                'back': 'setup_hostname',
                'next': None
            }
        ]
        self.configure()

    def configure(self):
        """Инициализация переменных"""
        self.hostname = hostname_util.load_hostname(self.hostname_filename)
        self.isExit = False

    def setup_hostname(self):
        """
        Шаг 1: Настройка имени хоста
        """
        code, tag = hostname_util.setup_hostname_dialog(self.d, self.hostname)
        if code == hostname_util.DIALOG_FORWARD:
            log.info("Новое имя хоста %s" % tag[0])
            self.hostname = tag[0]
            self.nextForDialog("setup_hostname")
        elif code == hostname_util.DIALOG_CLOSE:
            log.info("Завершение настройки имени хоста")
            hostname_util.close()

    def finish(self):
        """Шаг 2: Диалог завершения настройки"""
        code = hostname_util.finishDialog(self.d, self.hostname)
        if code == self.d.OK:
            self.saveConfig()
            log.info("Настройки успешно сохранены")
            log.info("Завершение настройки имени хоста")
            self.d.msgbox("Настройки успешно сохранены", height=6)
            hostname_util.close()
        else:
            self.backForDialog("finish")


    def saveConfig(self):
        """Записать файл описания сетевых интерфейсов"""
        hostname_util.save_hostname(self.hostname, self.hostname_filename, self.hosts_filename)
 
def main():
    log.info("Настройка имени хоста")
    try:
        wizard = HostnameWizard()
        wizard.run()
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение настройки имени хоста")
        pass
    log.info("Завершение настройки имени хоста")
    subprocess.call("clear")

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    main()
