#!/usr/bin/env python3

import sys
import os
import subprocess
import re
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger
from dialog import Dialog

log = logger.Logger()

# коды кнопок wizard'а
DIALOG_BACK = Dialog.EXTRA
DIALOG_FORWARD = Dialog.OK
DIALOG_CLOSE = Dialog.CANCEL
DIALOG_SKIP = Dialog.HELP

# лейблы кнопок wizard'а
DIALOG_OK_LABEL = "Далее"
DIALOG_EXTRA_LABEL = "Назад"
DIALOG_CANCEL_LABEL = "Закрыть"
DIALOG_SKIP_LABEL = "Пропустить"


def load_hostname(filename):
    """
    Загрузить hostname из файла filename
    """
    log.info("Загружаем имя хоста из файла %s" % filename)
    hostname = ""
    if os.path.exists(filename):
        fd = open(filename)
        lines = fd.readlines()
        fd.close()
        for line in lines:
            if line.strip():
                hostname = line
                break
    log.info("Текущее имя хоста: %s" % hostname)
    return hostname

def getValidHostname(hostname):
    """
    Преобразовать имя хоста в допустимый формат
    """
    log.info("Преобразование имени хоста: %s" % hostname)
    resultHostname = re.sub("[^0-9a-zA-Z\.-]",'',hostname,0)

    if len(resultHostname) > 64 or len(resultHostname) < 2:
        log.error("Недопустимая длина имени хоста")
        return None

    log.info("Преобразованное имя хоста: %s" % resultHostname)
    return resultHostname

def save_hostname(hostname, hostname_filename, hosts_filename):
    """
    Сохранить hostname в файл filename
    """

    log.info("Сохранение имени хоста в файл %s" % hosts_filename)
    hostname = getValidHostname(hostname)

    if not hostname:
        return

    log.info("Новое имя хоста: %s" % hostname)

    old_hostname = load_hostname(hostname_filename)
    if old_hostname != hostname:
        tmp_filename = "/tmp/hostconfig_hostname"
        fd = open(tmp_filename, "w")
        fd.write(hostname)
        fd.close()
        subprocess.call("sudo mv " + tmp_filename + " " + hostname_filename, shell=True)

        tmp_hosts = "/tmp/hostconfig_hosts"
        sourceFile = open(hosts_filename, "r")
        resultFile = open(tmp_hosts, "w")

        replaced = False
        for line in sourceFile:
            match = re.findall("127.0.0.1\s+(\S+)", line)
            if len(match):
                if not replaced:
                    replaced = True
                    log.info("Запись '127.0.0.1\t%s" % hostname + "\tlocalhost' в файл %s" % tmp_hosts)
                    resultFile.write("127.0.0.1\t" + hostname + "\tlocalhost\n")
            else:
                log.info("Запись '%s" % line + "' в файл %s" % tmp_hosts)
                resultFile.write(line)

        if not replaced:
            log.info("Запись '127.0.0.1\t%s" % hostname + "\tlocalhost' в файл %s" % tmp_hosts)
            resultFile.write("127.0.0.1\t" + hostname + "\tlocalhost\n")

        log.info("Перемещение файла %s" % tmp_hosts + " в %s" % hosts_filename)
        subprocess.call("sudo mv " + tmp_hosts + " " + hosts_filename, shell=True)

        log.info("Вызов команды 'sudo /bin/hostname -F /etc/hostname'")
        subprocess.call("sudo /bin/hostname -F /etc/hostname", shell=True)

def isExitDialog(d):
    """Диалог выхода"""
    return d.yesno("Сохранить настройки и закрыть программу?")

def close():
    subprocess.call("clear")
    sys.exit(0)

def setup_hostname_dialog(d, hostname):
    """
    Диалог настройки имени хоста
    """
    return d.form(
        text="Введите имя хоста",
        height=9, width=60, form_height=1,
        elements=[("Имя хоста:", 1, 1, hostname, 1, 12, 20, 60)],
        title="Шаг 1: Настройка имени хоста",
        ok_label=DIALOG_OK_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL
    )

def finishDialog(d, hostname):
    """Диалог завершения настройки"""
    text = "Настройка завершена\n" +\
        "имя хоста: " + hostname

    return d.msgbox(
        text,
        width=60,
        height=7,
        title="Шаг 2: Заверешение настройки",
        ok_label="Завершить",
        help_button="enable",
        help_label=DIALOG_EXTRA_LABEL,
    )
