#!/usr/bin/env python3

import dialog
import os
import subprocess
import sys

sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger

log = logger.Logger()

def create_dialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Выбор интерфейса"))
    return d


def createChooseInterfaceDialog(d):
    """
    Создать диалог выбора интерфейса
    """
    interfaces = [('keyboard', 'Обычный интерфейс', 1),
                  ('touch', 'Сенсорный интерфейс', 0)]

    (code, tag) = d.radiolist(
        'Выберите тип интерфейса кассы',
        width=60, height=len(interfaces) + 7,
        choices=interfaces)

    if code == d.OK:
        return True, tag
    else:
        return False, None


def writeInterface(interface):
    """
    Записать тип интерфейса в конфигурационный файл
    """
    confFile = '/linuxcash/cash/conf/ncash.ini.d/gui.ini'
    if not os.path.isfile(confFile):
        log.error('Файл %s не существует' % confFile)
        return

    oldValue = None
    with open(confFile, 'r') as file:
        for line in file:
            if 'interface' in line and '=' in line:
                oldValue = line.replace('\n', '')

    if oldValue is None:
        log.error('Настройка "interface" не найдена в файле %s' % confFile)
        return

    newValue = 'interface = ' + interface
    shell = subprocess.Popen("sed -i 's/%s/%s/g' %s" % (oldValue, newValue, confFile), shell=True,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    shell.communicate()


def main():
    """
    Главная функция
    """
    log.info("Выбор интерфейса")
    d = create_dialog()
    (res, interface) = createChooseInterfaceDialog(d)
    if not res:
        exit(0)

    writeInterface(interface)
    log.info("Завершение выбора интерфейса")


if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        main()
    except KeyboardInterrupt:
        log.warn('Неожиданное завершение редактирования выбора интерфейса')
        exit(1)
