#!/usr/bin/env python3

import dialog
import network_util
import os
import subprocess
import sys
import traceback
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger

log = logger.Logger()

def mainMenu(d):
    """
    Главное меню
    """
    return d.menu(
        "Управление соединениями",
        width=60, height=11, menu_height=4,
        choices=[
            ("manual", "Ручная настройка сети"),
            ("info", "Сетевые адреса"),
            ("quit", "Выйти")
        ])

def manualMenu(d, eths):
    """
    Меню ручной настройки
    """
    return d.menu(
        "Интерфейс: " + eths['selected'],
        width=60, height=16, menu_height=9,
        choices=[
            ("choose", "Выбрать сетевую карту"),
            ("edit", "Настройка сетевых параметров"),
            ("connect", "Подключить соединение"),
            ("disconnect", "Отключить соединение"),
            ("ping", "Проверить соединение"),
            ("source", "Посмотреть содержимое interfaces"),
            ("save", "Сохранить"),
            ("quit", "Выйти")
        ])

def chooseDeviceDialog(d, devices):
    """
    Диалог выбора сетевого интерфейса
    """
    return d.radiolist(
        "Какую сетевую карту требуется настроить?",
        width=60, height=len(devices) + 7,
        choices=devices)

def manualSetup(d, eths):
    """
    Настройка выбранного интерфейса eths статически
    """
    while True:
        (code, tag) = manualSetupDialog(d, eths)
        if code != d.OK:
            return False
        #проверяем введенные настройки
        if len(tag[0]) > 0:
            if len(tag[1]) == 0:
                tag[1] = "255.255.255.0"
            try:
                eths['network']['ethernets'][eths['selected']]['nameservers'] = {}
                eths['network']['ethernets'][eths['selected']]['nameservers']['addresses'] = tag[3].split(' ')
                eths['network']['ethernets'][eths['selected']]['gateway4'] = tag[2]
                eths['network']['ethernets'][eths['selected']]['addresses'] = [tag[0] + '/' + str(network_util.netmask_to_cidr(tag[1]))]
                eths['network']['ethernets'][eths['selected']]['dhcp4'] = False
                log.info("Интерфейс %s" % eths['selected'] + " успешно настроен \n%s" % eths['network']['ethernets'][eths['selected']])
                return True
            except:
                log.error("Ошибка синтаксиса маски интерфейса")
                d.msgbox("Ошибка синтаксиса маски интерфейса", height=6)
        else:
            log.warn("Ошибка валидности адреса интерфейса")
            d.msgbox("Вы должны ввести адрес интерфейса", height=6)

def manualSetupDialog(d, eths):
    """
    Диалог настройки интерфейса статически
    """
    network_util.createInterfaceIfEmpty(eths)
    eth = eths.get('network', {}).get('ethernets', {}).get(eths['selected'], {})
    ip, mask = network_util.cidr_to_netmask(eth['addresses'][0] if 'addresses' in eth and len(eth['addresses']) else '0.0.0.0/24')
    return d.form(
        text = "Введите параметры сетевого подключения",
        height=11, width=54, form_height=4,
        elements = [("IP-адрес:", 1, 1, ip, 1, 13, 25, 30),
            ("Маска сети:", 2, 1, mask, 2, 13, 25, 30),
            ("Шлюз:", 3, 1, eth['gateway4'] if 'gateway4' in eth else '', 3, 13, 25, 30),
            ("Список DNS:", 4, 1, ' '.join(eth['nameservers'].get('addresses', {}) if 'nameservers' in eth else {}), 4, 13, 25, 64)],
        title = "Настройка сети вручную")

def chooseEth(d, eths):
    """
    Выбор сетевого интерфейса из списка интерфесов eths
    """
    devsForDialog = []
    for dev in eths['interfaces']:
        if dev['name'] == eths['selected']:
            devsForDialog.append((dev['name'], dev['description'], 1))
        else:
            devsForDialog.append((dev['name'], dev['description'], 0))
    (code, tag) = chooseDeviceDialog(d, devsForDialog)
    if code == d.OK:
        eths['selected'] = tag

def pingDialog(d, ping_settings):
    """
    Диалог проверки соединения
    """
    if ping_settings['ip'] is None:
        ping_settings['ip'] = ""
    (code, tag) = d.form(
        text = "Введите параметры для проверки командой ping",
        height=10, width=54, form_height=2,
        elements = [("IP-адрес:", 1, 1, ping_settings['ip'], 1, 17, 16, 60),
            ("Кол-во пакетов:", 2, 1, ping_settings['count'], 2, 17, 16, 15)],
        title = "Проверка соединения")
    if code == d.OK:
        try:
            log.info("Старт проверки соединения")
            d.infobox("Подождите. Идет проверка соединения", height=7)
            ping_settings['ip'] = tag[0]
            ping_settings['count'] = tag[1]
            (code, (stdout, stderr)) = network_util.ping(ping_settings['ip'], ping_settings['count'])
            if code == 0:
                d.scrollbox(stdout, width=70, height=14)
            else:
                if len(stdout) > 0:
                    d.scrollbox(stdout)
                elif len(stderr) > 0:
                    d.scrollbox(stderr)
                else:
                    log.warn("Ошибка проверки соединения")
                    d.msgbox("Ошибка проверки соединения", height=6)
            log.info("Окончание проверки соединеиня")
        except KeyboardInterrupt:
            log.warn("Проверка соединения была прервана")
            d.msgbox("Проверка соединения была прервана", height=7)

def connectDialog(d, eths):
    """
    Диалог подключения выбранного интерфейса в eths
    """
    try:
        d.infobox("Подождите. Идет подключение соединения", height=7)
        network_util.disconnectNetplan(eths['selected'])
        (code, (stdout, stderr)) = network_util.connectNetplan(eths['selected'])
        if code == 0:
            log.info("Интерфейс %s" % eths['selected'] + ". Соединение успешно установлено")
            d.msgbox("Соединение успешно установлено", height=7)
        else:
            log.warn("Интерфейс %s" % eths['selected'] + ". Соединение не установлено")
            d.msgbox("Соединение не установлено", height=7)
    except KeyboardInterrupt:
        log.warn("Подключение было прервано")
        d.msgbox("Подключение было прервано", height=7)

def disconnectDialog(d, eths):
    """
    Диалог отключения выбранного интерфейса в eths
    """
    try:
        d.infobox("Подождите. Идет отключение соединения", height=8)
        network_util.disconnectNetplan(eths['selected'])
        d.msgbox("Соединение разорвано", height=7)
    except KeyboardInterrupt:
        log.warn("Отключение было прервано")
        d.msgbox("Отключение было прервано", height=7)

def typeOfSetupDialog(d):
    """
    Диалог выбора типа настройки сетевого интерфейса
    """
    return d.radiolist(
        text = "Выберите способ настройки интерфейса:",
        height=10, width=54,
        choices = [("auto", "Автоматическая настройка сети", 0),
            ("manual", "Настройка сети вручную", 1)])

def manualSubMenu(d):
    ping_settings = {
        'ip': None,
        'count': "4"
    }
    network_util.loadDialog(d)
    eths = { 'interfaces': network_util.getNetworkDevices() }
    isSaved = False
    isChanged = False
    interfaces_file = "/etc/netplan/01-netcfg.yaml"

    if len(eths['interfaces']) == 0:
        network_util.noInterfaceDialog(d)
        return

    eths['selected'] = eths['interfaces'][0]['name']
    eths.update(network_util.readNetplanConfig(eths, interfaces_file))

    while True:
        (code, tag) = manualMenu(d, eths)
        if code != d.OK:
            break
        if tag == "choose":
            log.info("Выбор сетевой карты")
            chooseEth(d, eths)
            ping_settings['ip'] = None
            log.info("Сетевая карта выбрана %s" % eths['selected'])
        elif tag == "edit":
            log.info("Настройка сетевых параметров")
            (code, tag) = typeOfSetupDialog(d)
            if code == d.OK:
                if tag == "manual":
                    log.info("Выбран способ 'вручную'")
                    isChanged = manualSetup(d, eths)
                elif tag == "auto":
                    log.info("Выбран способ 'автоматически'")
                    isChanged = True
                    network_util.createInterfaceIfEmpty(eths)
                    if eths['selected'] in eths['network']['ethernets']:
                        del eths['network']['ethernets'][eths['selected']]
                        eths['network']['ethernets'][eths['selected']] = {}
                    else:
                        eths['network']['ethernets'].update({eths['selected']: {}})
                    eths['network']['ethernets'][eths['selected']]['dhcp4'] = True
            log.info("Завершение настройки сетевых параметров")
        elif tag == "connect":
            log.info("Подключение соединения")
            connectDialog(d, eths)
            log.info("Выход из пункта 'Подключение соединения'")
        elif tag == "disconnect":
            log.info("Отключение соединения")
            disconnectDialog(d, eths)
            log.info("Выход из пункта 'Отключение соединения'")
        elif tag == "ping":
            log.info("Проверка соединения")
            pingDialog(d, ping_settings)
            log.info("Выход из пункта 'Проверка соединения'")
        elif tag == "source":
            log.info("Просмотр содержимого %s" % interfaces_file)
            if os.path.isfile(interfaces_file):
                d.textbox(interfaces_file)
            else:
                d.msgbox('Не найден файл %s' % interfaces_file, width=40, height=6)
            log.info("Выход из пункта 'Просмотр содержимого'")
        elif tag == "save":
            log.info("Сохранение настроек в конфигурационный файл")
            if isChanged == True:
                network_util.saveNetplanConfig(eths, interfaces_file)
                d.msgbox("Настройки сохранены")
                isSaved = True
            else:
                log.info("Настройки не были изменены")
                d.msgbox("Настройки не были изменены")
            log.info("Выход из пункта 'Cохранение настроек в конфигурационный файл'")
        else:
            if isSaved == False and isChanged == True:
                code = d.yesno("Вы не сохранили настройки. Хотите их сохранить?")
                if code == d.OK:
                    network_util.saveNetplanConfig(eths, interfaces_file)
                    log.info("Сохранение настроек для сетевой карты:\n%s" % eths['selected'] + "\nв файл %s" % interfaces_file)
            break

def main():
    """
    main function
    """
    log.info("Конфигурирование сети")
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Настройка сети"))
    while True:
        (code, tag) = mainMenu(d)
        if code != d.OK:
            break
        if tag == "manual":
            log.info("Ручная настройка сети")
            manualSubMenu(d)
            log.info("Завершение ручной настройки сети")
        elif tag == "info":
            log.info("Просмотр сетевых адресов")
            d.msgbox(network_util.getIPs())
            log.info("Завершение просмотра сетевых адресов")
        else:
            break
    log.info("Завершение конфигурирования сети")
    subprocess.call("clear")

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        main()
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение конфигурирования сети")
    except Exception:
        log.error(traceback.format_exc())
