#!/usr/bin/env python3

import lsb_release
import os
import re
import socket
import struct
import subprocess
import sys
import traceback
import yaml
sys.path.append("/linuxcash/cash/tools/tools_avail")
from dialog import Dialog
from py_tools.logger import logger

log = logger.Logger()

#коды кнопок wizard'а
DIALOG_BACK = Dialog.EXTRA
DIALOG_FORWARD = Dialog.OK
DIALOG_CLOSE = Dialog.CANCEL
DIALOG_SKIP = Dialog.HELP

#лейблы кнопок wizard'а
DIALOG_OK_LABEL = "Далее"
DIALOG_EXTRA_LABEL = "Назад"
DIALOG_CANCEL_LABEL = "Закрыть"
DIALOG_SKIP_LABEL = "Пропустить"

is_trusty_tuhr = lsb_release.get_distro_information()['RELEASE'] == '14.04'

#Функции для работы с сетью

def getIPs():
    """Получение IP адресов для всех интерфейсов"""
    log.info("Получение списка IP адресов для всех интерфейсов")
    result = ""
    process = subprocess.Popen(["ifconfig"], stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
    (stdout, stderr) = process.communicate()
    eths = re.split("\n\n", stdout)
    for eth in eths:
        if len(eth):
            matches = re.findall("(^[^\s]*)\s+(.|\n)*?inet %s([^\s]*)\s" % ('addr:' if is_trusty_tuhr else ''), eth)
            if len(matches) and len(matches[0]) > 2:
                if len(result):
                    result = result + "\n"
                result = result + matches[0][0] + "%s " % (":" if is_trusty_tuhr else "") + matches[0][2]
    log.info("Список получен")
    log.info("Список IP адресов:\n%s" % result)
    return result

def getNetworkDevices():
    """Получить список сетевых устройств в системе"""
    log.info("Получение списка сетевых устройств в системе")
    process = subprocess.Popen(["sudo", "lshw", "-C", "network"], stdout = subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    (stdout, stderr) = process.communicate()
    lshw_devices = re.split("\s*\*-network", stdout)[1:]
    network_devices = []

    for lshw_device in lshw_devices:
        if len(lshw_device) == 0:
            continue
        type = re.findall("(?<=description:\s).*(?=\sinterface)", lshw_device)
        descr = re.findall("(?<=product:\s).*", lshw_device)
        device = re.findall("(?<=logical name:\s).*", lshw_device)
        if not ( len(type) and len(device) and len(type[0]) and len(device[0])
                and not str(device[0]).startswith("tap") ):
            continue
        descr_iface = descr[0] if len(descr) and len(descr[0]) else ""
        eth = {
            'type': type[0],
            'name': device[0],
            'description': descr_iface,
            }
        if is_trusty_tuhr:
            eth['config'] = {
                'address': None,
                'netmask': None,
                'gateway': None,
                'mode': None,
                'ssid': None,
                'scan': None,
                'security': None,
                'pass': None,
                'psk': None
                }
        network_devices.append(eth)
        log.info("Найдено устройство:\n%s" % eth)
    log.info("Список сетевых устройств получен")
    return network_devices

def checkProperties(eth):
    log.info("Проверка параметров подключения")
    result = True
    message = ""
    if not eth['config']['mode']:
        result = False
        message = "\nСпособ подключения."
    elif eth['config']['mode'] == "static":
        if not eth['config']['address']:
            result = False
            message = message + "\nIP-адрес."
        if not eth['config']['netmask']:
            result = False
            message = message + "\nМаска сети."
    if eth['type'] == "Wireless":
        if not eth['config']['ssid'] or len(eth['config']['ssid']) == 0:
            result = False
            message = message + "\nИмя точки доступа."
        if not eth['config']['scan'] or len(eth['config']['scan']) == 0:
            result = False
            message = message + "\nВидимость точки доступа."
        if not eth['config']['security'] or len(eth['config']['security']) == 0:
            result = False
            message = message + "\nТип защиты."
        if not (eth['config']['pass'] and len(eth['config']['pass']) > 0) and not (eth['config']['psk'] and len(eth['config']['psk']) > 0):
            result = False
            message = message + "\nПароль."
    return (result, message)

def getPropertiesFromFile(eth, filename):
    """Получить сетевые настройки для интерфейса eth из файла filename"""
    log.info("Получение настроек интерфейсов для сетевой карты %s" % eth['name'] + " из файла %s" % filename)
    lines = os.popen("awk -f get.awk " + filename + " device=" + eth['name']).read().split("\n")
    for line in lines:
        if len(line) > 0:
            args = line.rsplit(" ")
            if args[0].count("wpa"):
                eth['config']['pass'] = ""
                if args[0] == "wpa-ssid":
                    eth['config']['ssid'] = args[1]
                if args[0] == "wpa-ap-scan":
                    eth['config']['scan'] = 'visible' if args[1] == '1' else 'hidden'
                if args[0] == "wpa-proto":
                    if args[1] == 'WPA':
                        eth['config']['security'] = 'wpa'
                    if args[1] == 'RSN':
                        eth['config']['security'] = 'wpa2'
                    if args[1] == 'WPA RSN':
                        eth['config']['security'] = 'wpa/wpa2'
                if args[0] == "wpa-psk":
                    eth['config']['psk'] = args[1]
            else:
                eth['config'][args[0]] = args[1]
    log.info("Настройки получены:\n%s" % eth)

def writePropertiesToInterfacesFile(eth, inputFilename, outputFilename):
    """Записать сетевые настройки интерфейса eth в файл filename"""
    log.info("Запись сетевых настроек интерфейса %s" % eth['name'] + " в файл %s" % outputFilename)
    tmpFilename = "/tmp/netconfig.tmp"
    command = ""
    wireless = ""
    if eth['type'] == 'Wireless':
        secur = eth['config']['security']
        wireless = " wpa_driver=wext wpa_ssid=" + eth['config']['ssid'] + " wpa_key_mgmt=WPA-PSK"
        if eth['config']['scan'] == 'visible':
            wireless = wireless + " wpa_ap_scan=1"      
        else:
            wireless = wireless + " wpa_ap_scan=2"

        if secur == "wpa":
            wireless = wireless + " wpa_proto=WPA wpa_pairwise=TKIP wpa_group=TKIP"
        if secur == "wpa2":
            wireless = wireless + " wpa_proto=RSN wpa_pairwise=CCMP wpa_group=CCMP"
        if secur == "wpa/wpa2":
            wireless = wireless + " wpa_proto=\"WPA RSN\" wpa_pairwise=\"TKIP CCMP\" wpa_group=\"TKIP CCMP\""

        wireless = wireless + " wpa_psk="
        if eth['config']['pass']:
            process = subprocess.Popen(["wpa_passphrase", eth['config']['ssid'], eth['config']['pass']], stdout = subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
            (stdout, stderr) = process.communicate()
            reKey = re.search("(?<=\spsk=).*", stdout)
            if reKey:
                wireless = wireless + reKey.group()
        else:
            wireless = wireless + eth['config']['psk']

    if eth['config']['mode'] == "dhcp":
        command = "awk -f edit.awk " + inputFilename + " device=" + eth['name'] + " mode=dhcp" + wireless + " > " + tmpFilename
    elif eth['config']['mode'] == "static":
        command = "awk -f edit.awk " + inputFilename  + " device=" + eth['name'] + " mode=static" + wireless +\
                  " address=" + eth['config']['address'] +\
                  " netmask=" + eth['config']['netmask']
        if eth['config']['gateway'] is not None and len(eth['config']['gateway']) > 0:
            command = command + " gateway=" + eth['config']['gateway']
        command = command + "> " + tmpFilename
    subprocess.call(command, shell=True)
    subprocess.call("sudo mv " + tmpFilename + " " + outputFilename, shell=True)
    log.info("Настройки интерфейса \n%s" % eth + "\nзаписаны в %s" % outputFilename)

def connect(eth, filename):
    """Подключить интерфейс eth"""
    log.info("Попытка подключения интерфейса %s" % eth['name'])
    process = subprocess.Popen(['sudo', 'ifup', '-i', filename, eth['name']], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    return process.wait(), process.communicate()

def connectNetplan(ifaceName):
    """Подключить интерфейс eth"""
    log.info("Попытка подключения интерфейса %s" % ifaceName)
    process = subprocess.Popen(['sudo', 'ifconfig', ifaceName, 'up'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    return process.wait(), process.communicate()

def disconnect(eth, filename):
    """Отключить интерфейс eth"""
    log.info("Отключение интерфейса %s" % eth['name'])
    process = subprocess.Popen(['sudo', 'ifdown', '-i', filename, eth['name']], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    log.info("Интерфейс %s" % eth['name'] + " отключен")
    return process.wait(), process.communicate()

def disconnectNetplan(ifaceName):
    """Отключить интерфейс eth"""
    log.info("Отключение интерфейса %s" % ifaceName)
    process = subprocess.Popen(['sudo', 'ifconfig', ifaceName, 'down'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    return process.wait(), process.communicate()

def ping(ip, count):
    """Выполнить команду ping для узла ip с кол-вом пакетов count"""
    log.info("Пинг адреса %s" % ip + " с количеством пакетов %s" % count)
    process = subprocess.Popen(['ping', ip, '-c', count], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    return process.wait(), process.communicate()

def isConnected(mode, code, stdout, stderr):
    """Анализ кода возврата и стандартных потоков вывовода и ошибок на момент подключения соединения
       Возвращает True, если соединение подключилось
    """
    log.info("Проверка подключения соединения")
    if code == 0:
        if mode == 'static':
            if len(stderr) == 0:
                return True
        elif mode == 'dhcp':
            if "bound to" in stderr:
                return True
    return False


#Диалоги

def loadDialog(d):
    """Диалог загрузки"""
    d.infobox("Подождите. Программа выполняет инициализацию устройств", height=6)

def noInterfaceDialog(d):
    """Диалог об отсутствующих интерфейсах"""
    return d.msgbox("Не обнаружено ни одного сетевого интерфейса. Настройка сети невозможна", height=8)

def typeOfSetupDialog(d, eth):
    """Диалог выбора типа настройки сетевого интерфейса"""
    values = [("auto", "Автоматически", 0),
        ("manual", "Вручную", 0)]
    if eth['config']['mode'] == 'static':
        values[1] = values[1][:-1] + (1,)
    else:
        values[0] = values[0][:-1] + (1,)

    return d.radiolist(
        text = "Выберите способ настройки интерфейса:",
        title = "Шаг 2: Выбор способа настройки",
        height=9, width=54,
        choices = values,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)

def manualSetupDialog(d, config):
    """Диалог настройки интерфейса статически"""
    return d.form(
        text = "Введите параметры сетевого подключения",
        height=10, width=60, form_height=3,
        elements = [("IP-адрес:", 1, 1, config['address'] if config['address'] is not None else "", 1, 13, 16, 15),
            ("Маска сети:", 2, 1, config['netmask'] if config['netmask'] is not None else "", 2, 13, 16, 15),
            ("Шлюз:", 3, 1, config['gateway'] if config['gateway'] is not None else "", 3, 13, 16, 15)],
        title = "Шаг 2.2: Настройка сети вручную",
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)

def isExitDialog(d):
    """Диалог выхода"""
    return d.yesno("Сохранить настройки и закрыть программу?")

def autoSetup(d, selectedEth):
    """Настройка интерфейса selectedEth по DHCP"""
    selectedEth['config']['mode'] = "dhcp"

def manualSetup(d, selectedEth):
    """Настройка интерфейса selectedEth статически"""
    selectedEth['config']['mode'] = "static"
    currentConfig = {}
    code = None
    for i in selectedEth['config']:
        currentConfig[i] = selectedEth['config'][i]
    isReady = False
    while isReady == False:
        (code, tag) = manualSetupDialog(d, currentConfig)
        if code == d.OK:
            #проверяем введенные настройки
            currentConfig['address'] = tag[0]
            currentConfig['netmask'] = tag[1]
            currentConfig['gateway'] = tag[2]
            isReady = True
            if len(currentConfig['address']) > 0:
                if len(currentConfig['netmask']) == 0:
                    currentConfig['netmask'] = "255.255.255.0"
            else:
                log.warn("Ошибка валидности адреса интерфейса")
                d.msgbox("Вы должны ввести адрес интерфейса", height=6)
                isReady = False

            if isReady:
                for i in currentConfig:
                    selectedEth['config'][i] = currentConfig[i]
            log.info("Интерфейс %s" % selectedEth['name'] + " успешно настроен \n%s" % selectedEth)
        else:
            isReady = True
    return code

def chooseDeviceDialog(d, devices):
    """Диалог выбора сетевого интерфейса"""
    return d.radiolist(
        "Какую сетевую карту требуется настроить?",
        title = "Шаг 1: Выбор сетевой карты",
        width=60, height=len(devices) + 7,
        choices=devices,
        cancel_label=DIALOG_CANCEL_LABEL,
        ok_label=DIALOG_OK_LABEL)

def scanAccessPoints(eth):
    """Поиск всех видимых WiFi точек доступа"""
    process = subprocess.Popen(['sudo', 'ifconfig', eth['name'], "up"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    process.wait()
    process = subprocess.Popen(['sudo', 'iwlist', eth['name'], "scan"], stdout = subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    (stdout, stderr) = process.communicate()
    iwlist = re.split("Cell\s\d+\s", stdout)[1:]
    accessPointsList = {}

    for accessPoint in iwlist:
        essid = None
        level = 0
        reEssid = re.search("(?<=ESSID:\").*(?=\")", accessPoint)
        reLevel = re.search("(?<=Quality=).*?(?=\s)", accessPoint)
        if reEssid:
            essid = reEssid.group()
            if reLevel:
                wf, ws = reLevel.group().split("/")
                try:
                    level = 100 * int(wf) // int(ws)
                except:
                    level = 0
            if essid not in accessPointsList:
                accessPointsList[essid] = level
            elif accessPointsList[essid] < level:
                accessPointsList[essid] = level
    return accessPointsList

def chooseScanType(d, eth):
    """Шаг 1.1: Выбор типа вещания имени точки доступа для Wireless интерфейса"""
    scan = eth['config']['scan']
    if not scan:
        scan = 'visible'
    return d.radiolist(
        "Точка доступа скрытая или видимая?",
        title = 'Шаг 1.1: Выбор типа вещания имени',
        width=60, height=9,
        choices=[('visible', 'Видимая точка доступа', int(scan == 'visible')),
            ('hidden', 'Скрытая точка доступа', int(scan == 'hidden'))],
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)

def chooseAP(d, eth):
    """Шаг 1.2: Выбор точки доступа из всех найденных при сканировании"""
    d.infobox("Подождите. Программа выполняет поиск точек доступа.", height=6)
    aps = scanAccessPoints(eth)
    if not len(aps):
        d.msgbox("Не обнаружено ни одной точки доступа.", height=6)
        return DIALOG_FORWARD, ''
    aplist = []
    for ap in aps:
        aplist.append((ap,'Сигнал ' + str(aps[ap]) + '%', 0))
    return d.radiolist(
        "Выберите одну из доступных",
        title = 'Шаг 1.2: Выбор точки доступа',
        width = 60, height = 11,
        choices = aplist,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL )

def setESSID(d, eth):
    """Шаг 1.3: Установка имени точки доступа"""
    ssid = eth['config']['ssid']
    if not ssid:
        ssid = ''
    return d.inputbox(
        "Отредактируйте или оставьте без изменения",
        title = 'Шаг 1.3: Установка имени точки доступа',
        width = 60, height = 9,
        init = ssid,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL )

def chooseSecurity(d, eth):
    """Шаг 1.4: Установка типа защиты"""
    secur = eth['config']['security']
    if not secur:
        secur = 'wpa'   
    return d.radiolist(
        "Выберите тип защиты",
        title = 'Шаг 1.2: Установка типа защиты',
        width = 60, height = 11,
        choices = [('wpa', 'WPA шифрование', int(secur == 'wpa')),
                ('wpa2', 'WPA2 шифрование', int(secur == 'wpa2')),
                ('wpa/wpa2', 'Смешанный режим', int(secur == 'wpa/wpa2'))],
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL )

def setPassword(d, eth):
    """Шаг 1.5: Установка пароля"""
    passw = eth['config']['pass']
    if not passw:
        passw = ''
    return d.inputbox(
        "Укажите пароль",
        title = 'Шаг 1.5: Установка пароля',
        width = 60, height = 11,
        init = passw,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL )

def chooseEth(d, eths, selectedEth):
    """Выбор сетевого интерфейса из списка интерфесов eths"""
    devsForDialog = []
    for dev in eths:
        if dev == selectedEth:
            devsForDialog.append((dev['name'], dev['description'], 1))
        else:
            devsForDialog.append((dev['name'], dev['description'], 0))
    (code, tag) = chooseDeviceDialog(d, devsForDialog)
    if code == d.OK:
        for eth in eths:
            if eth['name'] == tag:
                return eth
    elif code == DIALOG_CLOSE:
        return None
    else:
        return selectedEth
    return None

def pingDialog(d, selectedEth, ping_ip):
    """Диалог проверки соединения"""
    if selectedEth['config']['mode'] == "static" and ping_ip is None:
        ping_ip = selectedEth['config']['address']
    if ping_ip is None:
        ping_ip = ""
    (code, tag) = d.form(
        text = "Введите адрес для проверки командой ping",
        height=9, width=60, form_height=1,
        elements = [("IP-адрес:", 1, 1, ping_ip, 1, 11, 16, 15)],
        title = "Шаг 4: Проверка соединения",
        ok_label = "Проверить",
        cancel_label = DIALOG_CANCEL_LABEL)
    return code, tag

def finishDialog(d, eth):
    """Диалог завершения настройки"""
    text = "Настройка завершена\n" +\
           "интерфейс: " + eth['name'] + "\n" +\
           "тип настройки: " + eth['config']['mode'] + "\n"
    if eth['config']['mode'] == 'static':
        text = text +\
               "ip-адрес: " + eth['config']['address'] + "\n" +\
               "маска сети: " + eth['config']['netmask'] + "\n"
        if eth['config']['gateway'] is not None and len(eth['config']['gateway']) > 0:
            text = text + "ip-адрес шлюза: " + eth['config']['gateway']
    if eth['type'] == 'Wireless':
        scan = "видимая" if eth['config']['scan'] == 'visible' else "скрытая"
        text = text + "имя точки доступа: " + eth['config']['ssid'] + "\n" +\
                "видимость точки доступа: " + scan + "\n" +\
                "защита: " + eth['config']['security']  + "\n"
        if not eth['config']['security'] == 'no':
            text = text + "пароль: " + eth['config']['pass']  + "\n"

    return d.msgbox(
        text,
        width=60,
        height=12,
        title = "Шаг 5: Заверешение настройки",
        ok_label = "Завершить",
        help_button = "enable",
        help_label = DIALOG_EXTRA_LABEL,
    )

def readNetplanConfig(eths, filename):
    log.info("Получение настроек интерфейсов для сетевых карт из файла %s" % filename)
    try:
        with open(filename, 'r') as configFile:
            try:
                return yaml.safe_load(configFile)
            except yaml.YAMLError as exc:
                log.error(str(exc))
    except Exception:
        log.error(traceback.format_exc())
    return {'network': {'ethernets': {}}}

def saveNetplanConfig(eths, filename):
    try:
        with open(filename, 'w') as outfile:
            yaml.dump({'network': eths['network']}, outfile, default_flow_style=False, allow_unicode=True)
    except Exception:
        log.error(traceback.format_exc())
    process = subprocess.Popen(['sudo', 'netplan', 'apply'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    return process.wait(), process.communicate()

def cidr_to_netmask(cidr):
    '''
    Получение ip-адреса и маски сети из строки вида XXX.XXX.XXX.XXX/YY
    Возвращает кортеж из двух элементов - адрес и маска в виде YYY.YYY.YYY.YYY
    '''
    network, net_bits = cidr.split('/')
    host_bits = 32 - int(net_bits)
    netmask = socket.inet_ntoa(struct.pack('!I', (1 << 32) - (1 << host_bits)))
    return network, netmask

def netmask_to_cidr(netmask):
    '''
    Получение количества бит из маски сети вида YYY.YYY.YYY.YYY
    '''
    return sum([bin(int(x)).count('1') for x in netmask.split('.')])

def createInterfaceIfEmpty(eths):
    '''
    Создание пустого словаря интерфейса, если он отсутствует в eths
    '''
    if 'ethernets' not in eths['network']:
        eths['network'].update({'ethernets': {eths['selected']: {}}})
    elif eths['selected'] not in eths['network']['ethernets']:
        eths['network']['ethernets'].update({eths['selected']: {}})
