#!/usr/bin/env python3

import network_util
import dialog
import subprocess
import os
import sys
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.wizard.wizard import Wizard
from py_tools.logger import logger

log = logger.Logger()

class NetworkWizard(Wizard):
	"""Класс визарда настройки сети"""

	def __init__(self, d):
		"""Конструктор визарда настройки сети"""
		Wizard.__init__(self)
		#сетевые настройки
		self.d = d
		self.filename = "/etc/network/interfaces"
		self.checkFilename = "/tmp/tmp_interfaces"
		network_util.loadDialog(self.d)
		self.eths = []
		self.selectedEth = None
		self.isSaved = False
		self.isChanged = False
		self.ping_ip = None
		#настройки wizard'а
		self.currentStep = 0
		self.oldStep = 0
		self.dialogs = [
			{
				'name': 'chooseEth',
				'function': lambda: self.chooseEth(),
				'back': None,
				'next': {
						'Ethernet': 'typeOfSetup',
						'Wireless': 'chooseScanType'
				}
			},
			{
				'name': 'chooseScanType',
				'function': lambda: self.chooseScanType(),
				'back': 'chooseEth',
				'next': {
						'visible':'chooseAP',
						'hidden':'setESSID'
				}
			},
			{
				'name': 'chooseAP',
				'function': lambda: self.chooseAP(),
				'back': 'chooseScanType',
				'next': 'setESSID'
			},
			{
				'name': 'setESSID',
				'function': lambda: self.setESSID(),
				'back': {
						'hidden':'chooseScanType',
						'visible':'chooseAP'
				},
				'next': 'chooseSecurity'
			},
			{
				'name': 'chooseSecurity',
				'function': lambda: self.chooseSecurity(),
				'back': 'setESSID',
				'next': 'setPassword',
			},
			{
				'name': 'setPassword',
				'function': lambda: self.setPassword(),
				'back': 'chooseSecurity',
				'next': 'typeOfSetup'
			},
			{
				'name': 'typeOfSetup',
				'function': lambda: self.typeOfSetup(),
				'back': {
						'Wireless':'chooseSecurity',
						'Ethernet':'chooseEth'
				},
				'next': {
					'auto': 'autoSetup',
					'manual': 'manualSetup'
				}
			},
			{
				'name': 'manualSetup',
				'function': lambda: self.manualSetup(),
				'back': 'typeOfSetup',
				'next': 'connect'
			},
				{
				'name': 'autoSetup',
				'function': lambda: self.autoSetup(),
				'next': 'connect'
			},
			{
				'name': 'connect',
				'function': lambda: self.connect(),
				'back': {
					'dhcp': 'typeOfSetup',
					'static': 'manualSetup'
				},
				'next': 'ping'
			},
			{
				'name': 'ping',
				'function': lambda: self.ping(),
				'back': {
					'dhcp': 'typeOfSetup',
					'static': 'manualSetup'
				},
				'next': 'finish'
			},
			{
				'name': 'finish',
				'function': lambda: self.finish(),
			    'back': {
				    'dhcp': 'typeOfSetup',
				    'static': 'manualSetup'
				},
				'next': None
			}
		]
		self.configure()

	def configure(self):
		"""Инициализация переменных"""
		self.eths = network_util.getNetworkDevices()
		if len(self.eths) == 0:
			network_util.noInterfaceDialog(self.d)
			self.isExit = True
		else:
			self.selectedEth = self.eths[0]
			network_util.getPropertiesFromFile(self.selectedEth, self.filename)
			self.isExit = False

	def chooseEth(self):
		"""Шаг 1: Выбор настраиваемого интерфейса"""
		log.info("Выбор сетевой карты")
		self.selectedEth = network_util.chooseEth(self.d, self.eths, self.selectedEth)
		if self.selectedEth:
			log.info("Сетевая карта выбрана %s" % self.selectedEth['name'])
			network_util.getPropertiesFromFile(self.selectedEth, self.filename)
			self.nextForDialog("chooseEth", self.selectedEth['type'])
		else:
			log.info("Выход из автоматической настройки")
			self.close()

	def chooseScanType(self):
		"""Шаг 1.1: Выбор типа вещания имени точки доступа для Wireless интерфейса"""
		log.info("Выбор типа вещания для точки доступа %s" % self.selectedEth['config']['ssid'])
		(code, tag) = network_util.chooseScanType(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Тип вещания точки доступа %s" % self.selectedEth['config']['ssid'] + " изменён с %s" % self.selectedEth['config']['scan'] + " на %s" % tag)
			self.selectedEth['config']['scan'] = tag
			self.nextForDialog("chooseScanType", tag)
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("chooseScanType")
		
	def chooseAP(self):
		"""Шаг 1.2: Выбор одной из найденных точек доступа"""
		log.info("Выбор точки доступа")
		(code, tag) = network_util.chooseAP(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Выбрана точка доступа %s" % tag)
			self.selectedEth['config']['ssid'] = tag
			self.nextForDialog("chooseAP")
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("chooseAP")

	def setESSID(self):
		"""Шаг 1.3: Установка имени точки доступа"""
		log.info("Установка имени точки доступа %s" % self.selectedEth['config']['ssid'])
		(code, tag) = network_util.setESSID(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Изменено имя точки доступа '%s" % self.selectedEth['config']['ssid'] + "' на '%s" % tag + "'")
			self.selectedEth['config']['ssid'] = tag
			self.nextForDialog("setESSID")
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("setESSID", self.selectedEth['config']['scan'])
		
	def setPassword(self):
		"""Шаг 1.5: Установка пароля"""
		log.info("Установка пароля для точки доступа %s" % self.selectedEth['config']['ssid'])
		(code, tag) = network_util.setPassword(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Пароль для точки доступа %s" % self.selectedEtg['config']['ssid'] + " изменён с %s" % eth['config']['pass'] + " на %s" % tag)
			self.selectedEth['config']['pass'] = tag
			self.nextForDialog("setPassword")
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("setPassword")
		
	def chooseSecurity(self):
		"""Шаг 1.4: Установка типа защиты"""
		log.info("Установка типа защиты для точки доступа %s" % self.selectedEth['config']['ssid'])
		(code, tag) = network_util.chooseSecurity(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Изменён тип защиты точки доступа %s" % self.selectedEth['config']['ssid'] + " с %s" % self.selectedEth['config']['security'] + " на %s" % tag)
			self.selectedEth['config']['security'] = tag
			self.nextForDialog("chooseSecurity")
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("chooseSecurity")
		
	def typeOfSetup(self):
		"""Шаг 2: Выбор способа настройки интерфейса"""
		log.info("Выбор способа настройки интерфейса")
		(code, tag) = network_util.typeOfSetupDialog(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			log.info("Выбран способ '%s" % tag + "'")
			self.nextForDialog("typeOfSetup", tag)
		elif code == network_util.DIALOG_CLOSE:
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("typeOfSetup", self.selectedEth['type'])

	def manualSetup(self):
		"""Шаг 2.1: Настройка интерфейса статически"""
		log.info("Настройка параметров интерфейса %s" % self.selectedEth['name'] + " статически")
		code = network_util.manualSetup(self.d, self.selectedEth)
		if code == network_util.DIALOG_FORWARD:
			self.nextForDialog("manualSetup")
			network_util.writePropertiesToInterfacesFile(self.selectedEth, self.filename, self.checkFilename)
			self.ping_ip = None
		elif code == network_util.DIALOG_CLOSE:
			self.close()
		else:
			self.backForDialog("manualSetup")

	def autoSetup(self):
		"""Шаг 2.1: Настройка интерфейса по DHCP"""
		log.info("Настройка интерфейса по DHCP")
		network_util.autoSetup(self.d, self.selectedEth)
		network_util.writePropertiesToInterfacesFile(self.selectedEth, self.filename, self.checkFilename)
		self.nextForDialog("autoSetup")
		self.ping_ip = None

	def connect(self):
		"""Шаг 3.1: Диалог подключения"""
		log.info("Подключение соединения")
		self.d.infobox("Подождите. Идет подключение соединения\n\nПрервать подключение можно комбинацией клавиш Ctrl+C", title="Шаг 3: Подключение", height=8)
		interrupted = False
		try:
			network_util.disconnect(self.selectedEth, self.checkFilename)
			(code, (stdout, stderr)) = network_util.connect(self.selectedEth, self.checkFilename)
		except KeyboardInterrupt:
			interrupted = True
			log.warn("Подключение соединения было прервано")

		if not interrupted and network_util.isConnected(self.selectedEth['config']['mode'], code, stdout, stderr):
			log.info("Соединение успешно установлено")
			self.isConnected = True
			self.d.msgbox("Соединение успешно установлено", height=7)
			self.nextForDialog("connect")
		else:
			log.warn("Интерфейс %s" % self.selectedEth['name'] + ". Соединение не установлено")
			self.isConnected = False
			code = self.d.yesno("Подключение не было установлено. Хотите продолжить?", yes_label="Да", no_label="Нет", height=8)
			if code == self.d.OK:
				self.nextForDialog("connect")
			else:
				self.backForDialog("connect", self.selectedEth['config']['mode'])

	def ping(self):
		"""Шаг 4: Диалог пинга"""
		log.info("Проверка соединения")
		code = self.d.yesno("Хотите проверить подключение к сети?", yes_label="Да", no_label="Нет", height=7)
		if code == self.d.OK:
			isExit = False
			while not isExit:
				try:
					(code, tag) = network_util.pingDialog(self.d, self.selectedEth, self.ping_ip)
					if code == self.d.OK:
						self.d.infobox("Подождите. Идет проверка соединения", height=5)
						self.ping_ip = tag[0]
						count = "4"
						(code_of_ping, (stdout, stderr)) = network_util.ping(self.ping_ip, count)
						if code_of_ping == 0:
							self.d.scrollbox(stdout, width=70, height=14, exit_label = "Закрыть")
						else:
							if len(stdout) > 0:
								self.d.scrollbox(stdout)
							elif len(stderr) > 0:
								self.d.scrollbox(stderr)
							else:
								self.d.msgbox("Ошибка проверки соединения", height=5)
					else:
						isExit = True
						self.nextForDialog("ping")
				except KeyboardInterrupt:
					log.warn("Проверка соединения прервана")
		else:
			self.nextForDialog("ping")

	def finish(self):
		"""Шаг 5: Диалог завершения настройки"""
		log.info("Завершение настройки")
		code = network_util.finishDialog(self.d, self.selectedEth)
		if code == self.d.OK:
			self.saveConfig(self.filename)
			log.info("Настройки успешно сохранены")
			self.d.msgbox("Настройки успешно сохранены", height=6)
			log.info("Выход из автоматической настройки")
			self.close()
		else:
			self.backForDialog("finish", self.selectedEth['config']['mode'])

	def saveConfig(self, filename):
		"""Записать файл описания сетевых интерфейсов"""
		network_util.writePropertiesToInterfacesFile(self.selectedEth, self.checkFilename, filename)
		if os.path.exists(self.checkFilename):
			os.remove(self.checkFilename)
			log.info("Временный файл %s" % self.checkFilename + " удалён")

def main():
	d = dialog.Dialog()
	d.add_persistent_args(("--backtitle", "Настройка сети"))
	try:
		wizard = NetworkWizard(d)
		wizard.run()
	except KeyboardInterrupt:
		log.warn("Неожиданное завершение настройки сети")
		pass
	subprocess.call("clear")

if __name__ == "__main__":
	os.chdir(os.path.dirname(os.path.abspath(__file__)))
	main()
