#!/usr/bin/env python3

"""
Утилита конфигурирования puppet
"""

import os
import subprocess
import sys
import puppet_util
sys.path.append("/linuxcash/cash/tools/tools_avail")
from dialog import Dialog
from py_tools.logger import logger

log = logger.Logger()

def main():
    log.info("Настройка Puppet")
    puppet_conf_file = "/etc/puppet/puppet.conf"
    tmp_puppet_conf_file = "/tmp/puppet.conf"
    
    log.info("Копирование файла конфигурации %s" % puppet_conf_file + " во временный файл %s" % tmp_puppet_conf_file)
    subprocess.call("cp " + puppet_conf_file + " " + tmp_puppet_conf_file, shell = True)

    server, port = puppet_util.getPuppetConf(tmp_puppet_conf_file)
    try:
        d = Dialog()
        d.add_persistent_args(("--backtitle", "Настройка Puppet"))
        run = True
        while run:
            (code, tag) = puppet_util.setServerPortDialog(d, server, port, True)
            
            if code:
                run = False
                continue
            
            text = "Сервер: " + tag[0] + \
                "\nПорт: " + tag[1]
                
            code = d.msgbox(
                text,
                width=38,
                height=12,
                title = "Завершение настройки",
                ok_label = "Завершить",
                help_button = "enable",
                help_label = "Назад"
            )
            if code == d.OK:
                server = tag[0]
                port = tag[1]
                puppet_util.setPuppetConf(tmp_puppet_conf_file, server, port)
                log.info("Копирование файла конфигурации %s" % tmp_puppet_conf_file + " в %s" % puppet_conf_file)
                retCode = subprocess.call("sudo cp " + tmp_puppet_conf_file + " " + puppet_conf_file, shell=True)
                if not retCode:
                    log.info("Настройки успешно сохранены")
                    d.msgbox("Настройки успешно сохранены", height = 6)
                    run = False
                else:
                    log.error("Не удалось сохранить настройки")
                    d.msgbox("Не удалось сохранить настройки", height = 6)
        log.info("Завершение настройки Puppet")
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение настроки Puppet")
        exit(0)

    finally:
        if os.path.exists(tmp_puppet_conf_file):
            log.info("Удаление временного файла %s" % tmp_puppet_conf_file)
            subprocess.call("rm " + tmp_puppet_conf_file, shell=True)

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    main()
