#!/usr/bin/env python3

import os
import subprocess
import sys
import configparser
import lsb_release
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger

log = logger.Logger()

def getPuppetConf(filename):
    log.info("Получение настроек Puppet из файла %s" % filename)
    config = configparser.ConfigParser()
    config.read(filename)
    section = getSection()
    server = config.get(section, "server") if config.has_option(section, "server") else ""
    port = config.get(section, "masterport") if config.has_option(section, "masterport") else "8140"
    log.info("Настройки получены\nСервер: %s" % server + "\nПорт:%s" % port)
    return server, port

def setPuppetConf(filename, server, port):
    config = configparser.ConfigParser()
    config.read(filename)
    section = getSection()
    if not config.has_section(section):
        config.add_section(section)
    config.set(section, "server", server)
    config.set(section, "masterport", port)
    
    log.info("Запись настроек Puppet в файл %s" % filename + "\nСервер:%s" % server + "\nПорт:%s" % port)
    with open(filename, 'w') as conf_file:
        config.write(conf_file)
    log.info("Настройки записаны")

def setCertName(certName):
    certName = certName.lower()
    filename = '/etc/puppet/puppet.conf'
    config = configparser.ConfigParser()
    config.read(filename)
    config.set('main', "certname", certName)
    log.info("Запись имени сертификата %s" % certName + " в %s" % filename)
    with open(filename, 'w') as conf_file:
        config.write(conf_file)
    log.info("Имя сертификата записано")

def getSection():
    if not os.path.exists("/usr/sbin/puppetd") and os.path.exists("/usr/bin/puppet"):
        return "agent"
    return "puppetd"

def startService():
    log.info("Старт сервиса puppet")
    if lsb_release.get_distro_information()['CODENAME'] != 'trusty':
        os.system('systemctl enable puppet')
        os.system('systemctl start --no-block puppet')
    else:
        if os.path.exists('/etc/default/puppet'):
            os.system("sed -i '{s/^START[[:space:]]*=.*/START=yes/}' /etc/default/puppet")
        os.system('service puppet start --no-wait')

#Диалоги
def setServerPortDialog(d, server, port, firstrun=False):
    """Диалог редактирования параметров Puppet"""
    return d.form(
        text="Установите параметры Puppet",
        height=9, width=38, form_height=2,
        elements=[("Сервер:", 1, 1, server, 1, 9, 23, 100), ("Порт:", 2, 1, port, 2, 9, 23, 100)],
        ok_label="Далее" if firstrun else "Ok",
        cancel_label="Закрыть" if firstrun else "Отмена")

def removeSslSert(d):
    """Диалог удаления сертификатов SSL"""
    code = d.yesno("Удалить SSL сертификаты?",
                   yes_label="Да",
                   height=6, width=30,
                   no_label="Нет")
    if code == d.OK:
        log.info("Удаление сертификатов(директория /var/lib/puppet/ssl)")
        retCode = subprocess.call("sudo rm -rf /var/lib/puppet/ssl", shell=True)
        if not retCode:
            log.info("SSL сертификаты успешно удалены")
            d.msgbox("SSL сертификаты успешно удалены", height=6)
        else:
            log.error("Не удалось удалить SSL сертификаты")
            d.msgbox("Не удалось удалить SSL сертификаты", height=6)
