#!/usr/bin/env python3

import os
import subprocess
import sys
import puppet_util
sys.path.append("/linuxcash/cash/tools/tools_avail")
from dialog import Dialog
from py_tools.logger import logger

log = logger.Logger()

def main():
    log.info("Настройка Puppet")
    puppet_conf_file = "/etc/puppet/puppet.conf"
    tmp_puppet_conf_file = "/tmp/puppet.conf"
    
    log.info("Копирование файла конфигурации %s" % puppet_conf_file + " во временный файл %s" % tmp_puppet_conf_file)
    subprocess.call("cp " + puppet_conf_file + " " + tmp_puppet_conf_file, shell = True)
    
    server, port = puppet_util.getPuppetConf(tmp_puppet_conf_file)
    try:
        d = Dialog()
        d.add_persistent_args(("--backtitle", "Настройка Puppet"))
        isExit = False
        while not isExit:
            (m_code, m_tag) = d.menu(
                    "Выберите действие:",
                    width=60, height=11, menu_height=4,
                    choices = [("server", "Параметры доступа к серверу"),
                               ("ssl", "Удалить SSL сертификаты"),
                               ("start", "Обновление конфигурации"),
                               ("quit", "Выйти")])
            if m_code == d.OK:
                if m_tag == "server":
                    (code, tag) = puppet_util.setServerPortDialog(d, server, port)
                    if code == d.OK:
                        server = tag[0]
                        port = tag[1]
                        puppet_util.setPuppetConf(tmp_puppet_conf_file, server, port)
                        log.info("Копирование временного файла %s" % tmp_puppet_conf_file + " в файл конфигурации %s" % puppet_conf_file)
                        retCode = subprocess.call("sudo cp " + tmp_puppet_conf_file + " " + puppet_conf_file, shell=True)
                        if retCode:
                            log.error("Не удалось сохранить настройки")
                            d.msgbox("Не удалось сохранить настройки", height = 6)
                elif m_tag == "ssl":
                    puppet_util.removeSslSert(d)
                elif m_tag == "start":
                    log.info("Обновление конфигурации")
                    subprocess.call("/linuxcash/cash/tools/tools_avail/artix_configure.sh", shell=True)
                    log.info("Завершение обновления конфигурации")
                elif m_tag == "quit":
                    isExit = True
            else:
                isExit = True
        log.info("Завершение настройки Puppet")
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение настройки Puppet")
        exit(0)

    finally:
        if os.path.exists(tmp_puppet_conf_file):
            log.info("Удаление временного файла %s" % tmp_puppet_conf_file)
            subprocess.call("rm " + tmp_puppet_conf_file, shell=True)
            
if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    main()
