#!/usr/bin/env python3

import dialog
import os
import sys
import subprocess
import re
from copy import deepcopy
from time import sleep
import lsb_release
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.logger import logger

log = logger.Logger()

auto_master_file = "/etc/auto.master"
tmp_auto_master_file = "/tmp/auto.master"
auto_cifs_file = "/etc/auto.cifs"
tmp_auto_cifs_file = "/tmp/auto.cifs"
credentials_file = "/etc/auto.auth"
tmp_credentials_file = "/tmp/auto.auth"
is_trusty_tuhr = lsb_release.get_distro_information()['RELEASE'] == '14.04'

def create_dialog():
    """
    Создать объект диалога
    """
    d = dialog.Dialog()
    d.add_persistent_args(("--backtitle", "Подключение общего ресурса"))
    return d

def menu(d):
    """
    Меню выбора действия
    """
    return d.menu(
        "Выберите действие",
        width=40, height=12, menu_height=5,
        choices=[
            ("1", "Настроить"),
            ("2", "Отключить"),
            ("3", "Выход")
        ]
    )

def show_edit_form(d, sharedfolder_object):
    """
    Вывести форму редактирования настроек подключения общего ресурса
    """
    text = "Заполните параметры подключения общего ресурса\n"

    tag = []
    code, tag = d.form(
            text=text,
            height=12, width=38, form_height=4,
            elements=[
                   ("Сервер:", 1, 1, sharedfolder_object['hostname'], 1, 15, 17, 0),
                   ("Ресурс:", 2, 1, sharedfolder_object['resource'], 2, 15, 17, 0),
                   ("Пользователь:", 3, 1, sharedfolder_object['username'], 3, 15, 17, 0),
                   ("Пароль:", 4, 1, sharedfolder_object['password'], 4, 15, 17, 0)
            ],
            ok_label = "Далее",
            cancel_label = "Закрыть"
        )
    if code == d.OK:
        parse_object_from_tag(sharedfolder_object, tag)
    
    return code, tag

def parse_object_from_tag(sharedfolder_object, tag):
    """
    Заполнить поля sharedfolder_object из tag, который вернула форма
    """
    sharedfolder_object['hostname'] = tag[0]
    sharedfolder_object['resource'] = tag[1]
    sharedfolder_object['username'] = tag[2]
    sharedfolder_object['password'] = tag[3]


def backupFile(filename, tmpfilename):
    log.info("Копирование файла %s" % filename + " в %s" % tmpfilename)
    subprocess.call("cp " + filename + " " + tmpfilename, shell = True)
    if not os.path.exists(tmpfilename):
        log.info("Не удалось скопировать файл в %s" % tmpfilename)
        log.info("Создание файла %s" % tmpfilename)
        subprocess.call("touch " + tmpfilename, shell = True)

def writeFile(fileName, data, mode = 'a'):
    log.info("Запись данных в файл %s" % fileName)
    tmpFile = open(fileName, mode)
    tmpFile.write(data)
    tmpFile.close()

def readFile(fileName):
    log.info("Чтение данных из файла %s" % fileName)
    tmpFile = open(fileName)
    data = tmpFile.read()
    tmpFile.close()
    return data

def saveSettings(sharedfolder_object):
    log.info("Сохранение параметров в конфигурационные файлы")
    content_auto_master = readFile(tmp_auto_master_file)
    itselfIncluded = findSubstring(r'(\+auto.master)', content_auto_master)
    data = str("/linuxcash/net %s -v --timeout=300\n") % auto_cifs_file
    if itselfIncluded == "":
        data = '+auto.master\n' + data
    writeFile(tmp_auto_master_file, data)

    data = str("username=%s\npassword=%s") % (sharedfolder_object['username'], sharedfolder_object['password'])
    writeFile(tmp_credentials_file, data)

    sec_arg = "sec=ntlm," if is_trusty_tuhr else ""
    data = str("server -fstype=cifs,rw,noperm,%snocase,noacl,dir_mode=0777,file_mode=0666,credentials=%s ://%s/%s\n") 
    data = data % (sec_arg,credentials_file,sharedfolder_object['hostname'],sharedfolder_object['resource'])
    writeFile(tmp_auto_cifs_file, data)

    log.info("Перемещение файла %s" % tmp_auto_master_file + " в файл %s" % auto_master_file)
    retCode = subprocess.call("sudo mv " + tmp_auto_master_file + " " + auto_master_file, shell=True) 
    if not retCode:
        log.info("Перемещение файла %s" % tmp_auto_cifs_file + " в файл %s" % auto_cifs_file)
        retCode = subprocess.call("sudo mv " + tmp_auto_cifs_file + " " + auto_cifs_file, shell=True)
    else:
        log.warn("Не удалось переместить файл %s" % tmp_auto_master_file)
    if not retCode:
        log.info("Перемещение файла %s" % tmp_credentials_file + " в файл %s" % credentials_file)
        retCode = subprocess.call("sudo mv " + tmp_credentials_file + " " + credentials_file, shell=True)
    else:
        log.warn("Не удалось переместить файл %s" % tmp_auto_cifs_file)

    if retCode:
        log.warn("Не удалось переместить файл %s" % credentials_file)

    return retCode

def findSubstring(regExpr, data):
    result = re.findall(regExpr, data)
    value = ""
    try:
        value = str(result[0])
    except:
        log.error("Ошибка получения строкового значения")
        pass
    return value

def parseSettings(sharedfolder_object):
    log.info("Чтение параметров из конфигурационных файлов")
    if os.path.exists(auto_cifs_file):
        log.info("Чтение параметров 'hostname' и 'resource' из %s" % auto_cifs_file)
        data = readFile(auto_cifs_file)
        sharedfolder_object['hostname'] = findSubstring(r'://(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/', data)
        sharedfolder_object['resource'] = findSubstring(r'/(\w*)$', data)
    else:
        log.warn("Файл %s" % auto_cifs_file + " не обнаружен")

    if os.path.exists(credentials_file):
        log.info("Чтение параметров 'username' и 'password' из %s" % credentials_file)
        data = readFile(credentials_file)
        sharedfolder_object['username'] = findSubstring(r'username=(.*)', data)
        sharedfolder_object['password'] = findSubstring(r'password=(.*)', data)
    else:
        log.warn("Файл %s" % credentials_file + " не обнаружен")
    log.info("Параметры получены")

def mount(d, sharedfolder_object):
    tmp_sharedfolder_object = deepcopy(sharedfolder_object)
    retCode = 0
    run = True
    while run:
        code, tag = show_edit_form(d, sharedfolder_object)

        if code == d.OK:
            text = "Сервер: " + sharedfolder_object['hostname'] + \
                "\nРесурс: " + sharedfolder_object['resource'] + \
                "\nИмя пользователя: " + sharedfolder_object['username'] + \
                "\nПароль: " + sharedfolder_object['password']

            code = d.msgbox(
                text,
                width=38,
                height=12,
                title = "Завершение настройки",
                ok_label = "Завершить",
                help_button = "enable",
                help_label = "Назад"
            )
            if code == d.OK:
                backupFile(credentials_file, tmp_credentials_file)
                backupFile(auto_master_file, tmp_auto_master_file)
                backupFile(auto_cifs_file, tmp_auto_cifs_file)

                removeSettingsFromConfFiles(tmp_auto_cifs_file, tmp_auto_master_file, tmp_credentials_file,
                                            tmp_sharedfolder_object)
                retCode = saveSettings(sharedfolder_object)
                subprocess.call("service autofs restart", shell = True)
                sleep(2)
                if not retCode and isMountedSuccessfully():
                    log.info("Общий ресурс успешно подключен")
                    d.msgbox("Общий ресурс успешно подключен", height = 6)
                    run = False
                else:
                    log.error("Не удалось подключить общий ресурс")
                    d.msgbox("Не удалось подключить общий ресурс", height = 6)
                    tmp_sharedfolder_object = deepcopy(sharedfolder_object)
        else:
            run = False
    return retCode

def unmount(d, sharedfolder_object):
    removeSettingsFromConfFiles(auto_cifs_file, auto_master_file, credentials_file, sharedfolder_object)
    log.info("Общий ресурс успешно отключен")
    d.msgbox("Общий ресурс успешно отключен", height = 6)
    log.info("Рестарт autofs")
    subprocess.call("service autofs restart", shell = True)
    return 0

def removeSettingsFromConfFiles(cifs_file, master_file, credent_file, sharedfolder_object):
    log.info("Удаление параметров из конфигурационных файлов")
    if os.path.exists(credent_file):
        log.info("Удаление параметров из %s" % credent_file)
        writeFile(credent_file, '', 'w')
    else:
        log.warn("Файл %s" % credent_file + " не найден")

    if os.path.exists(master_file):
        log.info("Удаление параметров из %s" % master_file)
        data = readFile(master_file)
        p = re.compile(str("(/linuxcash/net %s -v --timeout=300[\n\r])") % auto_cifs_file)
        data = p.sub( '', data)
        writeFile(master_file, data, 'w')
    else:
        log.warn("Файл %s" % master_file + " не найден")

    if os.path.exists(cifs_file):
        log.info("Удаление параметров из %s" % cifs_file)
        data = readFile(cifs_file)
        substrings = re.findall('server -fstype=cifs,rw,noperm.*', data)
        for line in substrings:
            data = data.replace(line, '')
        writeFile(cifs_file, "%s\n" % data.strip(), 'w')
    else:
        log.warn("Файл %s" % cifs_file + " не найден")
    log.info("Удаление параметров завершено")

def isMountedSuccessfully():
    if os.path.exists("/linuxcash/net/server"):
        return True
    return False

def quit(d, sharedfolder_object):
    log.info("Завершение подключения общего ресурса")
    subprocess.call("clear", shell=True)
    sys.exit(0)

functions = {
    '1': mount,
    '2': unmount,
    '3': quit
}

def main(param):
    """
    Главная функция
    """
    log.info("Подключение общего ресурса")
    sharedfolder_object = {
        'hostname': "",
        'resource': "",
        'username': "",
        'password': ""
    }
    
    d = create_dialog()
    parseSettings(sharedfolder_object)
    
    result = 0
    if param == "firstrun":
        result = mount(d, sharedfolder_object)
    else:
        # Если выполняем в режиме maintainer, то задаем дополнительный диалог выбора действия
        (code, tag) = menu(d)
        if code == d.OK:
            if tag in functions:
                result = functions[tag](d, sharedfolder_object)
    log.info("Завершение подключения общего ресурса")
    subprocess.call("clear", shell=True)
    sys.exit(result)

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    try:
        if len(sys.argv) > 1 and ('firstrun' in sys.argv[1] or 'maintain' in sys.argv[1]):
            main(sys.argv[1])
        else:
            print("Необходим параметр: firstrun или maintain")
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение подключения общего ресурса")
        sys.exit(1)
